/******************************************************
*
* copyright by H.G.Kleta, 2002, 2006
*
* if you wish to use these sources, please contact
*
* fhw at indigo2 dot de
*
******************************************************/

#ifndef __MT_H_
#define __MT_H_


/******************************************************
*
* includes
*
******************************************************/

#include <math.h>
#include <stdio.h>

#include "mt_types.h"
#include "point_list.h"
#include "triangle_list.h"
#include "edge_list.h"


/******************************************************
*
* constant values
*
******************************************************/

#define COS_90_DEGREES         0.0
#define COS_120_DEGREES       -0.5

#define COS_MAX_ANGLE          COS_120_DEGREES
#define COS_SMALL_ANGLE        COS_90_DEGREES

#define MAX_EDGE_LENGTH_FACTOR 1.4
#define MIN_EDGE_LENGTH_FACTOR 0.7

#define STANDARD_BOUNDING_SPHERE    1
#define LONG_EDGE_BOUNDING_SPHERE   2
#define SMALL_ANGLE_BOUNDING_SPHERE 3


/******************************************************
*
* Coord f(Coord x, Coord y, Coord z)
*
* function returns distance of point p to isosurface
*
* various implementations...
*
******************************************************/

Coord f1(Coord x, Coord y, Coord z);
Coord f2(Coord x, Coord y, Coord z);
Coord f3(Coord x, Coord y, Coord z);
Coord f4(Coord x, Coord y, Coord z);
Coord f5(Coord x, Coord y, Coord z);
Coord f6(Coord x, Coord y, Coord z);
Coord f7(Coord x, Coord y, Coord z);
Coord f8(Coord x, Coord y, Coord z);
Coord f9(Coord x, Coord y, Coord z);


/******************************************************
*
* void NormalizeVector(Vector3D v, Vector3D result)
*
* function calculates normalized version of v
*
*          return: result = vector with lenght = 1 
*                           and direction from v
*
******************************************************/

void NormalizeVector(Vector3D v, Vector3D result);


/******************************************************
*
* void GetVectorProduct(Vector3D v1, Vector3D v2, Vector3D result)
*
* function caltulates crossproduct of v1 and v2
*
*          return: result = v1 x v2
*
******************************************************/

void GetVectorProduct(Vector3D v1, Vector3D v2, Vector3D result);


/******************************************************
*
* Coord GetScalarProduct(Vector3D v1, Vector3D v2)
*
* function calculates scalarproduct of v1 and v2
*
*          return: GetScalarProduct(v1,v2) = v1 . v2
*
******************************************************/

Coord GetScalarProduct(Vector3D v1, Vector3D v2);


/******************************************************
*
* Coord GetEdgeAngle(EdgeListElement *e1, EdgeListElement *e2)
*
* function calculates angle between e1 and e2
*
*          return: -1 : Angle > 180
*                  else cos(angle between e1 and e2)
*
******************************************************/

Coord GetEdgeAngle(EdgeListElement *e1, EdgeListElement *e2);


/******************************************************
*
* void GetGradient(Point3D p, Vector3D result)
*
* function calculates gradient of point p in scalar field
*
*          return: result = gradient of p in scalar field
*                           can be used as normal
*
******************************************************/

void GetGradient(Point3D p, Vector3D result);


/******************************************************
*
* void GetNormal(Point3D p1, Point3D p2, Point3D p3 , Vector3D result)
*
* function calculates normal of triangle described by p1,p2,p3
*
*          return: result = normal of triangle described by p1,p2,p3
*
******************************************************/

void GetNormal(Point3D p1, Point3D p2, Point3D p3, Vector3D result);


/******************************************************
*
* void GetNewPoint(Point3D i, Point3D j, Point3D k, Coord l, Point3D x)
*
* function calculates point x in plane i,j,k (counterclockwise ->
*          mathmatical positive viewed from top) perpendicular to edge i,j
*          with distance l from e (middle between i and j)
*
*          return: x
*
*             x
*            X
*            |
*           l|
*        j   |e    i
*       X    X    X
*
*               k
*              X
*
******************************************************/

void GetNewPoint(Point3D i, Point3D j, Point3D k, Coord l, Point3D x);


/******************************************************
*
* void MovePointOntoSurface(Point3D x)
*
* function moves point x onto isosurface (along gradient of x)
*
*          return: x
*
******************************************************/

void MovePointOntoSurface(Point3D x);


/******************************************************
*
* Coord GetPointDistance(Point3D p1, Point3D p2)
*
* function returns distance between p1 and p3
*
******************************************************/

Coord GetPointDistance(Point3D p1, Point3D p2);


/******************************************************
*
* int PointIsEqual(Point3D p1, Point3D p2)
*
* function checks p1 and p2 for equality
*
*          return: 0: p1 != p2
*                  1: p1 == p2
*
******************************************************/

int PointIsEqual(Point3D p1, Point3D p2);


/******************************************************
*
* void GetBoundingSphere(Point3D j, Point3D i, Point3D x,
*                        Point3D center, Coord *radius)
*
* function calculates bounding sphere of the triangle
*          described by j,i,x and returns the radius
*          and the center of the sphere
*
*          return: center = center of bounding sphere
*                  radius = radius of bounding sphere
*
*             x
*            X
*
*          center
*        j   X     i
*       X         X
*
******************************************************/

void GetBoundingSphere(Point3D j, Point3D i, Point3D x,
                       Point3D center, Coord *radius);


/******************************************************
*
* TriangleListElement * Create1stTriangle(Coord l);
*
* function creates first triangle needed for the 
*          marching triangle algorithm
*
*          return: pointer to triangle (list element)
*
******************************************************/

TriangleListElement * Create1stTriangle(Coord l);


/******************************************************
*
* TriangleListElement * MarchingTriangle(Coord l, int maxTriangleCount,
*                                        int fieldfunction);
*
* function returns pointer to trianngle list describing
*          trianglulated fieldfunction
*
*          l:                projection length of mt algorithm
*          maxTriangleCount: number of triangles to create (-1 = unlimited)
*          fieldfunction:    fieldfunction to be triangulated
*
******************************************************/

TriangleListElement * MarchingTriangle(Coord l, int maxTriangleCount,
                                                int fieldfunction);


/*****************************************************/

#endif /* __MT_H_ */

/* END_OF_FILE */
